#include <SDL.h>
#include <string>
#include <ctime>
#include <math.h>

const int SCR_WIDTH = 320;
const int SCR_HEIGHT = 320;
const int SCR_BPP = 32;
const bool TURN_PC = false;
const bool TURN_PLR = true;

struct loc {
	int x;
	int y;
};
	
SDL_Surface *screen = NULL;
SDL_Surface *x = NULL;
SDL_Surface *o = NULL;
SDL_Surface *board = NULL;
SDL_Surface *game_win = NULL;
SDL_Surface *game_lose = NULL;
SDL_Surface *game_tie = NULL;

SDL_Event event;

int ttt_board[3][3];

SDL_Surface* load_image(std::string);
void apply_surface(int x, int y, SDL_Surface* source, SDL_Surface* destination);

bool init();
bool load_files();
bool clean_up();
bool random_move();
int row_check();
int row_check_3(int f, int s, int t);
int play_game();
int winning_row();
int turn_row_on_off(int row, int onoff, int factor);
int show_board(int delay, int game_winner);
void update_board();

int main( int argc, char* args[] )
{
	srand(time(NULL));
	int who_won = 0;
	bool game_quit = false, input = false;

	if (init() == false) {
		return 1;
	}

	if (load_files() == false) {
		return 2;
	}
	
	while (game_quit == false) {
		who_won = play_game();
		input = false;
		while (input == false) {
			switch (who_won) {
				case 1:
					apply_surface(0, 0, game_tie, screen);
					break;
				case 2:
					apply_surface(0, 0, game_win, screen);
					break;
				case 3:
					apply_surface(0, 0, game_lose, screen);
					break;
				default:
					break;
			}

			while (SDL_PollEvent(&event)) {
				if (event.type == SDL_QUIT) {
					input = true;
					game_quit = true;
				} else if (event.type = SDL_MOUSEBUTTONDOWN) {
					if (event.button.button == SDL_BUTTON_LEFT) {
						if (event.button.x > 130 && event.button.x < 250) {
							if (event.button.y > 230 && event.button.y < 290) {
								input = true;
							}
						}
					}
				}
			}
			
			if(SDL_Flip(screen) == -1) {
				return 666;    
			}
			
		}
	}

	clean_up();
	
	// lol
	 
    return 0;    
}

bool init() {
	if (SDL_Init(SDL_INIT_EVERYTHING) == -1) {
		return false;
	}

	screen = SDL_SetVideoMode(SCR_WIDTH, SCR_HEIGHT, SCR_BPP, SDL_SWSURFACE);
	if (screen == NULL) {
		return false;
	}

	SDL_WM_SetCaption("Tic-Tact-Toe", NULL);

	return true;
}

bool load_files() {
	x = load_image("X.bmp");
	if (x == NULL) {
		return false;
	}

	o = load_image("O.bmp");
	if (o == NULL) {
		return false;
	}

	board = load_image("Bord.bmp");
	if (board == NULL) {
		return false;
	}

	game_win = load_image("Congratz.bmp");
	if (game_win == NULL) {
		return false;
	}

	game_lose = load_image("Fail.bmp");
	if (game_lose == NULL) {

	}

	game_tie = load_image("OhMyGawd.bmp");
	if (game_tie == NULL) {

	}

	return true;
}

int play_game() {
	// Init Vars
	bool quit = false, turn = true, r_move = true;
	int x = 0, y = 0;
	int board_x = 0, board_y = 0;
	int win;
	
	// Set all the board locations to zero
	for (int yT = 0; yT<3; yT++) {
		for (int xT = 0; xT<3; xT++) {
			ttt_board[xT][yT] = 0;
		}
	}
	
	while (quit == false) {
		while (SDL_PollEvent(&event)) {
			if (event.type == SDL_QUIT) {
				quit = true;
			} else if (event.type == SDL_MOUSEBUTTONDOWN) {
				if (event.button.button == SDL_BUTTON_LEFT) {
					// calculate mouse position to board grid
					x = event.button.x;
					y = event.button.y;

					board_x = x / 110;
					board_y = y / 110;
					
					if (ttt_board[board_x][board_y] == 0 && turn == TURN_PLR) {
						// if it's the players turn, set the current board box to 1
						ttt_board[board_x][board_y] = 1;
						turn = TURN_PC;
					}
				}
			}
		}

		// check for any three-in-a-rows.
		// zero for no ^^
		// 1 for x win
		// 2 for o win
		win = row_check();

		if (r_move == false && win == 0) {
			// tie
			show_board(4, 0);
			return 1;
		}

		if (win == 1) {
			// player won
			show_board(4, 1);
			return 2;
		}

		if (win == 2) {
			// pc won (zomg)
			show_board(4, 2);
			return 3;
		}

		if (turn == TURN_PC) {
			// make the computer do a random move
			r_move = random_move();
			turn = TURN_PLR;
		}

		update_board();

		if(SDL_Flip(screen) == -1) {
			return 999;    
		}

	}
	return 0;
}

int row_check() {
	// define variables for every box on the board
	int tl, t, tr;
	int ml, m, mr;
	int bl, b, br;
	
	// define variables for every possible 3-in-a-row combi
	int th, mh, bh;
	int lv, mv, rv;
	int dd, du;
	
	// fill all the variables with their boxes
	tl = ttt_board[0][0];
	t = ttt_board[1][0];
	tr = ttt_board[2][0];
	
	ml = ttt_board[0][1];
	m = ttt_board[1][1];
	mr = ttt_board[2][1];

	bl = ttt_board[0][2];
	b = ttt_board[1][2];
	br = ttt_board[2][2];
	
	// add the three boxes in one 3-in-a-row. if its three, player wins
	// if its 6, pc wins.
	// if its an other number, it isn;t a three-in-a-row
	th = row_check_3(tl, t, tr);
	mh = row_check_3(ml, m, mr);
	bh = row_check_3(bl, b, br);

	lv = row_check_3(tl, ml, bl);
	mv = row_check_3(t, m, b);
	rv = row_check_3(tr, mr, br);

	dd = row_check_3(tl, m, br);
	du = row_check_3(bl, m, tr);
	
	if (th == 1) {return 1;} else if (th == 2) {return 2;}
	if (mh == 1) {return 1;} else if (mh == 2) {return 2;}
	if (bh == 1) {return 1;} else if (bh == 2) {return 2;}
	if (lv == 1) {return 1;} else if (lv == 2) {return 2;}
	if (mv == 1) {return 1;} else if (mv == 2) {return 2;}
	if (rv == 1) {return 1;} else if (rv == 2) {return 2;}
	if (dd == 1) {return 1;} else if (dd == 2) {return 2;}
	if (du == 1) {return 1;} else if (du == 2) {return 2;}

	return 0;
}

int row_check_3(int f, int s, int t) {
	if (f == 1 && s == 1 && t == 1) {
		return 1;
	}

	if (f == 2 && s == 2 && t == 2) {
		return 2;
	}

	return 0;
}

bool random_move() {
	int counter1 = 0;
	int counter2 = 0;
	int move = 0;
	int pc_turn = 0;

	for (int yT = 0; yT<3; yT++) {
		for (int xT = 0; xT<3; xT++) {
			if (ttt_board[xT][yT] == 0) {
				counter1++;
			}
		}
	}

	loc * locations = new loc [counter1];

	for (int yT = 0; yT<3; yT++) {
		for (int xT = 0; xT<3; xT++) {
			if (ttt_board[xT][yT] == 0) {
				locations[counter2].x = xT;
				locations[counter2].y = yT;
				counter2++;
			}
		}
	}

	if (counter1 == 0) {
		return false;
	}

	pc_turn = rand() % counter1;

	ttt_board[locations[pc_turn].x][locations[pc_turn].y] = 2;

	return true;
}

bool clean_up() {
	SDL_FreeSurface(board);
	SDL_FreeSurface(o);
	SDL_FreeSurface(x);
	SDL_FreeSurface(game_win);
	SDL_FreeSurface(game_lose);
	SDL_FreeSurface(game_tie);

	SDL_Quit();
	
	return true;
}

void update_board() {
	apply_surface(0, 0, board, screen);

	for (int yT = 0; yT<3; yT++) {
		for (int xT = 0; xT<3; xT++) {
			if (ttt_board[xT][yT] == 1) {
				apply_surface(xT * 110, yT * 110, x, screen);
			}
			if (ttt_board[xT][yT] == 2) {
				apply_surface(xT * 110, yT * 110, o, screen);
			}
		}
	}
}

int show_board(int delay, int game_winner) {
	update_board();

	SDL_Flip(screen);

	int win_row;
	
	win_row = winning_row();

	clock_t wait[9];
	wait[0] = clock() + (delay * CLOCKS_PER_SEC);
	wait[1] = clock() + ((delay / (2*delay + 0.0)) * CLOCKS_PER_SEC);
	wait[2] = clock() + ((delay / (2*delay + 0.0)) * 2 * CLOCKS_PER_SEC);
	wait[3] = clock() + ((delay / (2*delay + 0.0)) * 3 * CLOCKS_PER_SEC);
	wait[4] = clock() + ((delay / (2*delay + 0.0)) * 4 * CLOCKS_PER_SEC);
	wait[5] = clock() + ((delay / (2*delay + 0.0)) * 5 * CLOCKS_PER_SEC);
	wait[6] = clock() + ((delay / (2*delay + 0.0)) * 6 * CLOCKS_PER_SEC);
	wait[7] = clock() + ((delay / (2*delay + 0.0)) * 7 * CLOCKS_PER_SEC);
	wait[8] = clock();

	while (clock() < wait[0]) {
		while (SDL_PollEvent(&event)) {
			if (event.type == SDL_QUIT) {
				return 1;
			}
		}
		
		if (clock() > wait[8]) {
			turn_row_on_off(win_row, 0, game_winner);
		}

		if (clock() > wait[1]) {
			turn_row_on_off(win_row, 1, game_winner);
		}

		if (clock() > wait[2]) {
			turn_row_on_off(win_row, 0, game_winner);
		}

		if (clock() > wait[3]) {
			turn_row_on_off(win_row, 1, game_winner);
		}
		
		if (clock() > wait[4]) {
			turn_row_on_off(win_row, 0, game_winner);
		}

		if (clock() > wait[5]) {
			turn_row_on_off(win_row, 1, game_winner);
		}

		if (clock() > wait[6]) {
			turn_row_on_off(win_row, 0, game_winner);
		}

		if (clock() > wait[7]) {
			turn_row_on_off(win_row, 1, game_winner);
		}

		update_board();

		SDL_Flip(screen);
	}

	return 0;
}

int turn_row_on_off(int row, int onoff, int factor) {
	switch (row) {
		case 1:
			ttt_board[0][0] = onoff * factor;
			ttt_board[1][0] = onoff * factor;
			ttt_board[2][0] = onoff * factor;
			break;
		case 2:
			ttt_board[0][1] = onoff * factor;
			ttt_board[1][1] = onoff * factor;
			ttt_board[2][1] = onoff * factor;
			break;
		case 3:
			ttt_board[0][2] = onoff * factor;
			ttt_board[1][2] = onoff * factor;
			ttt_board[2][2] = onoff * factor;
			break;
		case 4:
			ttt_board[0][0] = onoff * factor;
			ttt_board[0][1] = onoff * factor;
			ttt_board[0][2] = onoff * factor;
			break;
		case 5:
			ttt_board[1][0] = onoff * factor;
			ttt_board[1][1] = onoff * factor;
			ttt_board[1][2] = onoff * factor;
			break;
		case 6:
			ttt_board[2][0] = onoff * factor;
			ttt_board[2][1] = onoff * factor;
			ttt_board[2][2] = onoff * factor;
			break;
		case 7:
			ttt_board[0][0] = onoff * factor;
			ttt_board[1][1] = onoff * factor;
			ttt_board[2][2] = onoff * factor;
			break;
		case 8:
			ttt_board[2][0] = onoff * factor;
			ttt_board[1][1] = onoff * factor;
			ttt_board[0][2] = onoff * factor;
			break;
		default:
			break;
	}

	return 0;
}

int winning_row() {
	// define variables for every box on the board
	int tl, t, tr;
	int ml, m, mr;
	int bl, b, br;
	
	// fill all the variables with their boxes
	tl = ttt_board[0][0];
	t = ttt_board[1][0];
	tr = ttt_board[2][0];
	
	ml = ttt_board[0][1];
	m = ttt_board[1][1];
	mr = ttt_board[2][1];

	bl = ttt_board[0][2];
	b = ttt_board[1][2];
	br = ttt_board[2][2];

	if (tl == t && t == tr && tr == tl) {return 1;}
	if (ml == m && m == mr && mr == ml) {return 2;}
	if (bl == b && b == br && br == bl) {return 3;}
	if (tl == ml && ml == bl && bl == tl) {return 4;}
	if (t == m && m == b && t == b) {return 5;}
	if (tr == mr && mr == br && tr == br) {return 6;}
	if (tl == m && m == br && tl == br) {return 7;}
	if (tr == m && m == bl && bl == tr) {return 8;}

	return 0;
}

/*From here...*/

SDL_Surface* load_image(std::string filename) {
	SDL_Surface* loadedImage = NULL;
	SDL_Surface* optimizedImage = NULL;

	loadedImage = SDL_LoadBMP(filename.c_str());

	if (loadedImage != NULL) {
		optimizedImage = SDL_DisplayFormat(loadedImage);
		SDL_FreeSurface(loadedImage);
	}

	return optimizedImage;
}

void apply_surface(int x, int y, SDL_Surface* source, SDL_Surface* destination) {
	SDL_Rect offset;

	offset.x = x;
	offset.y = y;
	
	SDL_BlitSurface(source, NULL, destination, &offset);
}

/*
...to here the code was originally from Lazy Foo' Productions (http://lazyfoo.net/)
I hought: why would i reinvent the weel?
*/